@@ -38,7 +38,7 @@ class ProcessTerminationError(Exception):
     '''Indicates that the host process crashed while processing a command.'''
     def __init__(self, signal):
         self._signal_or_returncode = signal
-        message = 'Isolation host terminated with signal or returncode '+str(signal)
+        message = 'Isolated child process terminated with signal or returncode '+str(signal)
         Exception.__init__(self, message)
 
 class ClientStateError(Exception):
@@ -350,6 +350,7 @@ class CallableObjectProxy(ObjectProxy):
 class ExceptionProxy(Exception,ObjectProxy):
     def __init__(self, prime_id, prime_docstring=None):
         ObjectProxy.__init__(self, prime_id, prime_docstring)
+        Exception.__init__(self)
     def __reduce__(self):
         return ExceptionProxy, (self.prime_id,)
 
@@ -791,10 +792,12 @@ class Client(object):
 
         # Unpack any exception raised on the server side
         if isinstance(result, ExceptionalResult):
+            # Attach the exception, which may be a proxy to a server-side exception
+            if isinstance(result.exception, Proxy):
+                self.attach_proxy(result.exception)
+
             # TODO: append the server-side traceback to the client-side traceback
             logger.debug('client recieved exceptional result: %s', result)
-            #logger.debug('server-side traceback:')
-            #logger.debug(result.traceback)
             result.exception.message += 'Server-side traceback:\n' + result.traceback
             if isinstance(result.exception, BaseException):
                 raise result.exception
@@ -878,13 +881,24 @@ class IsolationContext(object):
     '''Represents a domain for executing code that is isolated from
     the rest of the process. Each isolation context corresponds to a
     single sub-process in which one or more python modules has been
-    loaded.'''
+    loaded.
+
+    An IsolationContext is constructed in an uninitialized
+    state. Calling start() will cause a child process to be created
+    and linked to this context. If that child process terminates, for
+    example due to an signal from the operating system, then the
+    isolation context will be permanently in a terminated state and
+    any attempt to access remote objects will cause a
+    ClientStateError. Isolation contexts cannot be re-started:
+    instead, create a new isolation context.'''
     def __init__(self):
         self._client = None
 
     @property
     def remote_pid(self):
-        '''Get the PID of the process in which this context will run code.'''
+        '''Get the operating system process ID of the process in which
+        this context will run code. This is equivalent to calling
+        os.getpid() in the child process.'''
         return self.client.server_process.pid
 
     @property
@@ -911,48 +925,37 @@ class IsolationContext(object):
         # Create a client to talk to the server
         self._client = Client(server_process, request_queue, response_queue)
 
-    def restart(self):
-        '''Terminate the child process and start a new process. This
-        does not restore any state, so any python modules that were
-        previously loaded will need to be reloaded.'''
-        logger.debug('IsolationContext[%d] restarting', id(self))
-        if self._client is not None:
-            # It is always safe to call cleanup no matter what state the client is in
-            self._client.cleanup()
-            self._client = None
-        self.start()
-
     def ensure_started(self):
         '''If the subprocess for this isolation context has not been created then create it.'''
         if self._client is None:
             self.start()
 
-    def load_module(self, module_name):
+    def load_module(self, module_name, path=None):
         '''Import a module into this isolation context and return a proxy for it.'''
         self.ensure_started()
-        mod = self.client.call(_load_module, module_name, sys.path)
+        if path is None:
+            path = sys.path
+        mod = self.client.call(_load_module, module_name, path)
         mod.__isolation_context__ = self
         return mod
 
         
 
 
-
-
 def default_context():
     if not hasattr(default_context, '_instance'):
         default_context._instance = IsolationContext()
     return default_context._instance
 
-def load_module(module_name):
+def load_module(module_name, path=None):
     '''Import a module into the default isolated context and return a
     reference to it. This does not import the module into sys.modules
     and it does not have any of the other effects of python's "import
     foo" syntax.'''
-    return default_context().load_module(module_name)
+    return default_context().load_module(module_name, path)
 
-def import_isolated(module_name, fromlist=[], level=-1):
+def import_isolated(module_name, fromlist=[], level=-1, path=None):
     '''Import an module into an isolated context as if with
     "__import__('module_name')"'''
-    sys.modules[module_name] = load_module(module_name)
+    sys.modules[module_name] = load_module(module_name, path=path)
     return __import__(module_name, fromlist=fromlist, level=level)
